@@ -14,6 +14,7 @@
 #include <cstring>
 #include <fcitx-config/iniparser.h>
 #include <fcitx-utils/capabilityflags.h>
+#include <fcitx-utils/fdstreambuf.h>
 #include <fcitx-utils/fs.h>
 #include <fcitx-utils/i18n.h>
 #include <fcitx-utils/key.h>
@@ -21,7 +22,7 @@
 #include <fcitx-utils/log.h>
 #include <fcitx-utils/macros.h>
 #include <fcitx-utils/misc.h>
-#include <fcitx-utils/standardpath.h>
+#include <fcitx-utils/standardpaths.h>
 #include <fcitx-utils/stringutils.h>
 #include <fcitx-utils/textformatflags.h>
 #include <fcitx/action.h>
@@ -39,6 +40,7 @@
 #include <fcntl.h>
 #include <glib-object.h>
 #include <glib.h>
+#include <istream>
 #include <libkkc/libkkc.h>
 #include <memory>
 #include <stdexcept>
@@ -388,12 +390,12 @@ KkcEngine::KkcEngine(Instance *instance)
 #endif
     kkc_init();
 
-    fs::makePath(stringutils::joinPath(
-        StandardPath::global().userDirectory(StandardPath::Type::PkgData),
-        "kkc/dictionary"));
-    fs::makePath(stringutils::joinPath(
-        StandardPath::global().userDirectory(StandardPath::Type::PkgData),
-        "kkc/rules"));
+    fs::makePath(
+        StandardPaths::global().userDirectory(StandardPathsType::PkgData) /
+        "kkc/dictionary");
+    fs::makePath(
+        StandardPaths::global().userDirectory(StandardPathsType::PkgData) /
+        "kkc/rules");
 
     // We can only create kkc object here after we called kkc_init().
     model_.reset(kkc_language_model_load("sorted3", NULL));
@@ -588,22 +590,18 @@ void KkcEngine::updateInputMode(InputContext *ic) { modeAction_->update(ic); }
 
 void KkcEngine::loadDictionary() {
     kkc_dictionary_list_clear(dictionaries_.get());
-    auto file = StandardPath::global().open(StandardPath::Type::PkgData,
-                                            "kkc/dictionary_list", O_RDONLY);
-    if (file.fd() < 0) {
+    auto file = StandardPaths::global().open(StandardPathsType::PkgData,
+                                             "kkc/dictionary_list");
+    if (!file.isValid()) {
         return;
     }
-    UniqueFilePtr fp(fdopen(file.fd(), "r"));
-    if (!fp) {
-        return;
-    }
-    file.release();
 
-    UniqueCPtr<char> buf;
-    size_t len = 0;
+    IFDStreamBuf buf(file.fd());
+    std::istream in(&buf);
+    std::string line;
 
-    while (getline(buf, &len, fp.get()) != -1) {
-        auto tokens = stringutils::split(stringutils::trim(buf.get()), ",");
+    while (std::getline(in, line)) {
+        auto tokens = stringutils::split(stringutils::trimView(line), ",");
 
         if (tokens.size() < 3) {
             continue;
@@ -652,10 +650,9 @@ void KkcEngine::loadDictionary() {
             constexpr auto len = sizeof(configDir) - 1;
             std::string realpath = path;
             if (stringutils::startsWith(path, "$FCITX_CONFIG_DIR/")) {
-                realpath =
-                    stringutils::joinPath(StandardPath::global().userDirectory(
-                                              StandardPath::Type::PkgData),
-                                          path.substr(len));
+                realpath = StandardPaths::global().userDirectory(
+                               StandardPathsType::PkgData) /
+                           path.substr(len);
             }
 
             auto userdict = makeGObjectUnique(reinterpret_cast<KkcDictionary *>(
@@ -677,11 +674,11 @@ void KkcEngine::loadRule() {
     if (!meta) {
         return;
     }
-    std::string basePath = stringutils::joinPath(
-        StandardPath::global().userDirectory(StandardPath::Type::PkgData),
-        "kkc/rules");
+    const auto basePath =
+        StandardPaths::global().userDirectory(StandardPathsType::PkgData) /
+        "kkc/rules";
     userRule_.reset(
-        kkc_user_rule_new(meta, basePath.c_str(), "fcitx-kkc", NULL));
+        kkc_user_rule_new(meta, basePath.string().c_str(), "fcitx-kkc", NULL));
 }
 
 std::string KkcEngine::subMode(const InputMethodEntry & /*entry*/,
